import io
import os
import logging
from PIL import Image
from celery import shared_task

from .aws import publish_to_aws

logger = logging.getLogger("tasks")


@shared_task(acks_late=True)
def process_images(config):
    """
    Creates multiple sizes of an image as stated in config["sizes"] and
    processes them as stated in config["compression"], config["quality"], and
    config["progressive"]. Calls function to upload to S3. Then deletes image.
    """
    # Open the image
    logger.info(
        "process_image.py: Opening local file at {}.".format(
            config["path"] + config["filename"]
        )
    )
    base_img = Image.open(config["path"] + config["filename"])
    upload_imgs = []

    # Create multiple sizes
    if "size" in config["sizes"][0]:
        logger.info("Creating multiple sizes.")
        for size in config["sizes"]:
            logger.info("Creating size: {}.".format(size["size"]))
            copy = base_img.copy()
            copy.thumbnail((size["size"], base_img.height), Image.ANTIALIAS)
            upload_imgs.append({"filename": size["filename"], "file": copy})

    # Or upload the one size
    else:
        logger.info("Processing single size.")
        upload_imgs.append(
            {"filename": config["sizes"][0]["filename"], "file": base_img}
        )

    # Process and upload to s3
    for img in upload_imgs:
        imgByteArr = io.BytesIO()
        img["file"].save(
            imgByteArr,
            format=config["format"],
            quality=config["quality"],
            optimize=config["compression"],
            progressive=config["progressive"],
        )

        logger.info("Uploading to S3...")
        publish_to_aws(img["filename"], imgByteArr.getvalue())

    # Delete temporary image
    logger.info(
        "Deleting temporary image file: {}.".format(
            config["path"] + config["filename"]
        )
    )
    os.remove(os.path.join(config["path"], config["filename"]))



import os
from django.http import Http404
from django.views.generic import TemplateView
from django.conf import settings
from s3imageservice.conf import settings as app_settings
from s3imageservice.utils.auth import secure


@secure
class CMSBaseView(TemplateView):
    model = None

    def setup(self, request, *args, **kwargs):
        if self.model:
            try:
                self.test_model_instance_exists(request, *args, **kwargs)
            except:
                raise Http404("No {} found.".format(self.model.__name__))

        return super().setup(request, *args, **kwargs)

    def test_model_instance_exists():
        pass

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        context["S3_ROOT"] = os.path.join(
            app_settings.AWS_S3_STATIC_ROOT, app_settings.S3_UPLOAD_ROOT
        )
        context["FILE_MB_LIMIT"] = app_settings.FILE_MB_LIMIT
        context["CMS_PAGE_SIZE"] = app_settings.CMS_PAGE_SIZE
        context["CMS_DEFAULT_SIZES"] = app_settings.CMS_DEFAULT_SIZES

        if self.request.user.is_authenticated:
            context["user"] = self.request.user
            context["user_id"] = self.request.user.pk
        else:
            context["user"] = None
            context["user_id"] = None

        return context



from s3imageservice.cms.views.base import CMSBaseView
from django.views.generic.base import RedirectView


class Upload(CMSBaseView):
    template_name = "s3imageservice/upload.html"

    def test_model_instance_exists(self, request, *args, **kwargs):
        pass

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context



from django.urls import include, path
from rest_framework import routers
from django.contrib import admin
from .viewsets.upload import UploadViewset
from .views import Detail, List, Upload

router = routers.DefaultRouter()
router.register(r"upload", UploadViewset, base_name="upload")

app_name = "cms"

urlpatterns = [
    path("", List.as_view(), name="list"),
    path("upload/", Upload.as_view(), name="upload"),
    path("detail/<upload>/", Detail.as_view(), name="detail"),
    path("api/", include(router.urls)),
]



# Generated by Django 2.2.5 on 2019-09-27 19:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('s3imageservice', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='upload',
            name='slug',
            field=models.CharField(default='', max_length=50, unique=True),
            preserve_default=False,
        ),
    ]



import json
import uuid
import os
from operator import itemgetter
from PIL import Image
from datetime import datetime
import logging
from rest_framework.views import APIView
from rest_framework.parsers import MultiPartParser
from rest_framework.decorators import parser_classes
from rest_framework.response import Response
from rest_framework.exceptions import UnsupportedMediaType, ParseError
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication

from s3imageservice.conf import settings
from s3imageservice.celery import process_images

logger = logging.getLogger("tasks")

FILE_SIZE_LIMIT = settings.FILE_MB_LIMIT * 1000 * 1000
FILE_TYPES = ["image/jpg", "image/jpeg", "image/png", "image/gif"]


@parser_classes((MultiPartParser,))
class ImageService(APIView):
    """
    Class to handle image uploads.
    """

    authentication_classes = (
        SessionAuthentication,
        settings.API_AUTHENTICATION_CLASS,
    )
    permission_classes = (IsAuthenticated, settings.API_PERMISSION_CLASS)

    def post(self, request, format=None):
        """
        Handles image uploads, processes them, and returns image and path data
        if successful.
        """

        try:
            file = request.FILES["img"]
        except KeyError:
            raise ParseError(detail="'img' must be provided.")

        sizes = request.data.get("sizes", None)
        if sizes:
            try:
                sizes = json.loads(sizes)
            except json.decoder.JSONDecodeError:
                raise ParseError(
                    detail="'sizes' must be a json-parsable array."
                )

        compression = request.data.get("compression", "")
        progressive = request.data.get("progressive", "")
        convertToJPG = request.data.get("convertToJPG", "")

        self.validation(file, sizes)

        resp_data = self.processform(
            file,
            sizes,
            compression=compression,
            progressive=progressive,
            convertToJPG=convertToJPG,
        )

        return Response(resp_data)

    def validation(self, file, sizes):
        """
        Validates an image based on predefined paramters of type and size.
        """
        if file.size > FILE_SIZE_LIMIT:
            raise UnsupportedMediaType(
                file.content_type,
                detail="'img' must be less than %sMB" % settings.FILE_MB_LIMIT,
            )

        if file.content_type not in FILE_TYPES:
            raise UnsupportedMediaType(
                file.content_type,
                detail="'img' must be of type(s) %s" % str(FILE_TYPES),
            )

        if sizes:
            if len(sizes) < 1:
                raise ParseError(detail="'sizes' must have at least one size.")

            if not isinstance(sizes, list):
                raise ParseError(
                    detail="'sizes' must be a json-parsable array."
                )

            for size in sizes:
                if not isinstance(size, int):
                    raise ParseError(
                        detail="'sizes' must be an array of integers."
                    )

        return True

    def processform(
        self,
        file,
        sizes,
        compression=None,
        progressive=None,
        convertToJPG=None,
    ):
        """
        Upload and creates images for each size provided.
        Returns response with image and path data.
        """

        # Get base file and filename
        base_img = Image.open(file)
        hash = uuid.uuid4().hex[0:10]
        logger.info("Request for image service approved: {}".format(hash))

        # conversion settings
        convertToJPG = True if convertToJPG.lower() == "true" else False
        format = base_img.format
        ext = format.lower()
        export_format = "JPEG" if convertToJPG or format == "JPEG" else format
        export_ext = "jpg" if convertToJPG else ext

        # Handle options
        if export_format == "JPEG":
            compression = False if compression.lower() == "false" else True
            progressive = False if progressive.lower() == "false" else True
            quality = 80 if compression else 100
        else:
            compression = False
            progressive = False
            quality = 100

        # Convert color options for PNGs
        if convertToJPG:
            base_img = base_img.convert("RGB")

        # Create multiple sizes
        images = []
        now = datetime.now()
        if sizes:
            for size in sizes:
                filename = "{y}/{m}/{d}/{hash}-{size}.{ext}".format(
                    y=now.year,
                    m=now.month,
                    d=now.day,
                    hash=hash,
                    size=size,
                    ext=export_ext,
                )
                images.append({"filename": filename, "size": size})
        else:
            filename = "{y}/{m}/{d}/{hash}.{ext}".format(
                y=now.year, m=now.month, d=now.day, hash=hash, ext=export_ext
            )
            images.append({"filename": filename})

        # Save image in local media
        file_path = os.path.join(settings.MEDIA_ROOT, settings.MEDIA_PATH)
        if not os.path.exists(file_path):
            os.makedirs(file_path)
        base_filname = "{hash}.{ext}".format(hash=hash, ext=export_ext)

        logger.info(
            "Saving local media file in {}".format(file_path + base_filname)
        )

        base_img.save(file_path + base_filname, format=export_format)
        config = {
            "format": export_format,
            "ext": export_ext,
            "path": file_path,
            "filename": base_filname,
            "compression": compression,
            "progressive": progressive,
            "quality": quality,
            "sizes": images,
        }
        process_images.delay(config)

        # Create response
        sorted_imgs = (
            sorted(images, key=itemgetter("size"))
            if "size" in images
            else images
        )

        base_url = os.path.join(
            settings.AWS_S3_STATIC_ROOT, settings.S3_UPLOAD_ROOT
        )

        resp_urls = [base_url + img["filename"] for img in sorted_imgs]

        canonical_url = "https://s3.amazonaws.com/" + os.path.join(
            settings.AWS_S3_BUCKET,
            settings.S3_UPLOAD_ROOT,
            sorted_imgs[-1]["filename"],
        )

        if sizes:
            # Multiple size response
            resp_sizes = [img["size"] for img in sorted_imgs]

            resp_srcset = ", ".join(
                [
                    "%s%s %iw" % (base_url, img["filename"], img["size"])
                    for img in sorted_imgs
                ]
            )

            resp_img_sizes = []
            for idx, img in enumerate(sorted_imgs):
                if idx != len(sorted_imgs) - 1:
                    resp_img_sizes.append(
                        "(max-width: {s}px) {s}px".format(s=img["size"])
                    )
                else:
                    resp_img_sizes.append("{s}px".format(s=img["size"]))
            resp_img_sizes = ", ".join(resp_img_sizes)

            if export_format == "JPEG":
                return {
                    "success": "ok",
                    "format": export_format,
                    "canonical": canonical_url,
                    "urls": resp_urls,
                    "sizes": resp_sizes,
                    "img": {
                        "srcset": resp_srcset,
                        "sizes": resp_img_sizes,
                        "src": resp_urls[-1],
                    },
                }
            else:
                return {
                    "success": "ok",
                    "format": export_format,
                    "canonical": canonical_url,
                    "urls": resp_urls,
                    "sizes": resp_sizes,
                    "img": {"src": canonical_url},
                }

        else:
            # Single size response
            return {
                "success": "ok",
                "format": export_format,
                "canonical": canonical_url,
                "urls": resp_urls,
                "img": {"src": canonical_url},
            }



# Generated by Django 2.2.5 on 2019-09-25 15:39

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Upload',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('canonical', models.URLField(unique=True)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                ('owner', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



import logging
import os

from s3imageservice.conf import settings
from s3imageservice.utils.aws import defaults, get_bucket
from celery import shared_task

logger = logging.getLogger('tasks')

OUTPUT_PATH = settings.S3_UPLOAD_ROOT
CACHE_HEADER = str('max-age=604800')  # 1 week


@shared_task(acks_late=True)
def publish_to_aws(filepath, data):

    key = os.path.join(
        OUTPUT_PATH,
        filepath
    )

    bucket = get_bucket()

    bucket.put_object(
        Key=key,
        ACL=defaults.ACL,
        Body=data,
        CacheControl=CACHE_HEADER,
        ContentType='image/jpg'
    )

    logger.info('AWS Published: %s.' % (key))



from s3imageservice.models import Upload
from s3imageservice.cms.serializers.upload import UploadSerializer
from s3imageservice.conf import settings
from django.core.paginator import Paginator
from rest_framework import viewsets, status
from rest_framework.views import APIView
from rest_framework.pagination import PageNumberPagination
from rest_framework.permissions import IsAuthenticated
from rest_framework.authentication import SessionAuthentication
from rest_framework.decorators import action
from rest_framework.response import Response


class DefaultPagination(PageNumberPagination):
    page_size = settings.CMS_PAGE_SIZE


class UploadViewset(viewsets.ModelViewSet):
    authentication_classes = ()
    permission_classes = ()
    pagination_class = DefaultPagination
    throttle_classes = []
    queryset = Upload.objects.all()
    serializer_class = UploadSerializer

    @action(detail=False, methods=["get"])
    def filter_for_user(self, request):
        user = request.GET.get("user", None)
        if not user:
            return Response(
                "No user provided.", status=status.HTTP_400_BAD_REQUEST
            )

        queryset = Upload.objects.filter(owner=user)

        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)
        else:
            return Response(
                "Something went wrong.",
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )
